'''
# AUTHORS:
# Sakib Mahmud, Texas A&M University, Geochemical and Environmental Research Group, sakib@tamu.edu
# Xiao Ge, Texas A&M University, Geochemical and Environmental Research Group, gexiao@tamu.edu
# Alec Krueger, Texas A&M University, Geochemical and Environmental Research Group, alecmkrueger@tamu.edu


Module to ingest and process raw glider data into NetCDF files
'''
from .MissionProcessor import MissionProcessor
from .MissionData import MissionData


import numpy as np
import pandas as pd
import xarray as xr
import dbdreader
import gsw
from attrs import define

from glider_ingest.MissionData import MissionData
from glider_ingest.utils import print_time


@define
class ScienceProcessor:

    mission_data:MissionData

    def get_sci_vars(self,variables:list):
        # Define subsets of columns based on the presence of sci_oxy4_oxygen and sci_flbbcd_bb_units
        if 'sci_oxy4_oxygen' in variables and 'sci_flbbcd_bb_units' in variables:
                present_variables = ['sci_flbbcd_bb_units', 'sci_flbbcd_cdom_units', 'sci_flbbcd_chlor_units', 'sci_water_pressure', 'sci_water_temp', 'sci_water_cond', 'sci_oxy4_oxygen']

        elif 'sci_oxy4_oxygen' in variables and 'sci_flbbcd_bb_units' not in variables:
                present_variables = ['sci_water_pressure', 'sci_water_temp', 'sci_water_cond', 'sci_oxy4_oxygen']

        elif 'sci_oxy4_oxygen' not in variables and 'sci_flbbcd_bb_units' in variables:
                present_variables = ['sci_flbbcd_bb_units', 'sci_flbbcd_cdom_units', 'sci_flbbcd_chlor_units', 'sci_water_pressure', 'sci_water_temp', 'sci_water_cond']

        elif 'sci_oxy4_oxygen' not in variables and 'sci_flbbcd_bb_units' not in variables:
                present_variables = ['sci_water_pressure', 'sci_water_temp', 'sci_water_cond']

        return present_variables

    def load_science(self):
        files = self.mission_data.get_files(files_loc=self.mission_data.sci_files_loc,extension='ebd')
        dbd = dbdreader.MultiDBD(files,cacheDir=self.mission_data.sci_cache_loc)

        all_variables = dbd.parameterNames['sci']
        present_variables = self.get_sci_vars(all_variables)
        vars = dbd.get_sync(*present_variables)

        self.mission_data.df_sci = pd.DataFrame(vars).T

        column_names = ['sci_m_present_time']
        column_names.extend(present_variables)

        self.mission_data.df_sci.columns = column_names

        self.mission_data.df_sci['sci_m_present_time'] = pd.to_datetime(self.mission_data.df_sci['sci_m_present_time'], unit='s', errors='coerce')
        self.mission_data.df_sci = self.mission_data.df_sci.dropna()

        # Remove any data with erroneous dates
        self.mission_data.df_sci = self.mission_data.df_sci.loc[(self.mission_data.df_sci['sci_m_present_time'] > self.mission_data.mission_start_date) & (self.mission_data.df_sci['sci_m_present_time'] < self.mission_data.mission_end_date)]

        # Convert pressure from db to dbar
        self.mission_data.df_sci['sci_water_pressure'] = self.mission_data.df_sci['sci_water_pressure'] * 10
        # Calculate salinity and density
        self.mission_data.df_sci['sci_water_sal'] = gsw.SP_from_C(self.mission_data.df_sci['sci_water_cond']*10,self.mission_data.df_sci['sci_water_temp'],self.mission_data.df_sci['sci_water_pressure'])
        CT = gsw.CT_from_t(self.mission_data.df_sci['sci_water_sal'],self.mission_data.df_sci['sci_water_temp'],self.mission_data.df_sci['sci_water_pressure'])
        self.mission_data.df_sci['sci_water_dens'] = gsw.rho_t_exact(self.mission_data.df_sci['sci_water_sal'],CT,self.mission_data.df_sci['sci_water_pressure'])

        self.mission_data.df_sci = self.mission_data.df_sci.dropna()
        dbd.close()
        return self.mission_data.df_sci

    def convert_sci_df_to_ds(self) -> xr.Dataset:
        '''Convert the given science dataframe to a xarray dataset'''
        platform_ds = xr.Dataset() # put the platform info into the dataset on the top
        platform_ds['platform'] = xr.DataArray(self.mission_data.glider_id)
        self.mission_data.ds_sci = xr.Dataset.from_dataframe(self.mission_data.df_sci)
        self.mission_data.ds_sci = platform_ds.update(self.mission_data.ds_sci)


    def add_sci_attrs(self) -> xr.Dataset:
        '''Add attributes to the science dataset'''
        variables = list(self.mission_data.ds_sci.data_vars)
        # Define variable attributes
        self.mission_data.ds_sci['platform'].attrs = {'ancillary_variables': ' ',
        'comment': ' ',
        'id': self.mission_data.glider_id,
        'instruments': 'instrument_ctd',
        'long_name': 'Slocum Glider '+ self.mission_data.glider_id,
        'type': 'platform',
        'wmo_id': self.mission_data.wmo_id,
        'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
        self.mission_data.ds_sci['sci_water_pressure'].attrs = {'accuracy': 0.01,
        'ancillary_variables': ' ',
        'axis': 'Z',
        'bytes': 4,
        'comment': 'Alias for sci_water_pressure, multiplied by 10 to convert from bar to dbar',
        'instrument': 'instrument_ctd',
        'long_name': 'CTD Pressure',
        'observation_type': 'measured',
        'platform': 'platform',
        'positive': 'down',
        'precision': 0.01,
        'reference_datum': 'sea-surface',
        'resolution': 0.01,
        'source_sensor': 'sci_water_pressure',
        'standard_name': 'sea_water_pressure',
        'units': 'bar',
        'valid_max': 2000.0,
        'valid_min': 0.0,
        'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
        self.mission_data.ds_sci['sci_water_temp'].attrs = {'accuracy': 0.004,
        'ancillary_variables': ' ',
        'bytes': 4,
        'instrument': 'instrument_ctd',
        'long_name': 'Temperature',
        'observation_type': 'measured',
        'platform': 'platform',
        'precision': 0.001,
        'resolution': 0.001,
        'standard_name': 'sea_water_temperature',
        'units': 'Celsius',
        'valid_max': 40.0,
        'valid_min': -5.0,
        'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
        self.mission_data.ds_sci['sci_water_cond'].attrs = {'accuracy': 0.001,
        'ancillary_variables': ' ',
        'bytes': 4,
        'instrument': 'instrument_ctd',
        'long_name': 'sci_water_cond',
        'observation_type': 'measured',
        'platform': 'platform',
        'precision': 1e-05,
        'resolution': 1e-05,
        'standard_name': 'sea_water_electrical_conductivity',
        'units': 'S m-1',
        'valid_max': 10.0,
        'valid_min': 0.0,
        'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
        self.mission_data.ds_sci['sci_water_sal'].attrs = {'accuracy': ' ',
        'ancillary_variables': ' ',
        'instrument': 'instrument_ctd',
        'long_name': 'Salinity',
        'observation_type': 'calculated',
        'platform': 'platform',
        'precision': ' ',
        'resolution': ' ',
        'standard_name': 'sea_water_practical_salinity',
        'units': '1',
        'valid_max': 40.0,
        'valid_min': 0.0,
        'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
        self.mission_data.ds_sci['sci_water_dens'].attrs = {'accuracy': ' ',
        'ancillary_variables': ' ',
        'instrument': 'instrument_ctd',
        'long_name': 'Density',
        'observation_type': 'calculated',
        'platform': 'platform',
        'precision': ' ',
        'resolution': ' ',
        'standard_name': 'sea_water_density',
        'units': 'kg m-3',
        'valid_max': 1040.0,
        'valid_min': 1015.0,
        'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
        if 'sci_flbbcd_bb_units' in variables:
            self.mission_data.ds_sci['sci_flbbcd_bb_units'].attrs = {'long_name':'science turbidity', 'standard_name':'backscatter', 'units':'nodim'}
            self.mission_data.ds_sci['sci_flbbcd_bb_units'].attrs = {'accuracy': ' ',
            'ancillary_variables': ' ',
            'instrument': 'instrument_flbbcd',
            'long_name': 'Turbidity',
            'observation_type': 'calculated',
            'platform': 'platform',
            'precision': ' ',
            'resolution': ' ',
            'standard_name': 'sea_water_turbidity',
            'units': '1',
            'valid_max': 1.0,
            'valid_min': 0.0,
            'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
            self.mission_data.ds_sci['sci_flbbcd_cdom_units'].attrs = {'accuracy': ' ',
            'ancillary_variables': ' ',
            'instrument': 'instrument_flbbcd',
            'long_name': 'CDOM',
            'observation_type': 'calculated',
            'platform': 'platform',
            'precision': ' ',
            'resolution': ' ',
            'standard_name': 'concentration_of_colored_dissolved_organic_matter_in_sea_water',
            'units': 'ppb',
            'valid_max': 50.0,
            'valid_min': 0.0,
            'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}
            self.mission_data.ds_sci['sci_flbbcd_chlor_units'].attrs = {'accuracy': ' ',
            'ancillary_variables': ' ',
            'instrument': 'instrument_flbbcd',
            'long_name': 'Chlorophyll_a',
            'observation_type': 'calculated',
            'platform': 'platform',
            'precision': ' ',
            'resolution': ' ',
            'standard_name': 'mass_concentration_of_chlorophyll_a_in_sea_water',
            'units': '\u03BCg/L',
            'valid_max': 10.0,
            'valid_min': 0.0,
            'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}

        if 'sci_oxy4_oxygen' in variables:
            self.mission_data.ds_sci['sci_oxy4_oxygen'].attrs = {'accuracy': ' ',
            'ancillary_variables': ' ',
            'instrument': 'instrument_ctd_modular_do_sensor',
            'long_name': 'oxygen',
            'observation_type': 'calculated',
            'platform': 'platform',
            'precision': ' ',
            'resolution': ' ',
            'standard_name': 'moles_of_oxygen_per_unit_mass_in_sea_water',
            'units': '\u03BCmol/kg',
            'valid_max': 500.0,
            'valid_min': 0.0,
            'update_time': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S')}

    def format_sci_ds(self) -> xr.Dataset:
        '''Format the science dataset by sorting and renameing variables'''
        self.mission_data.ds_sci['index'] = np.sort(self.mission_data.ds_sci['sci_m_present_time'].values.astype('datetime64[ns]'))
        self.mission_data.ds_sci = self.mission_data.ds_sci.drop_vars('sci_m_present_time')
        if 'sci_oxy4_oxygen' in self.mission_data.ds_sci.data_vars.keys():
            self.mission_data.ds_sci = self.mission_data.ds_sci.rename({'index': 'time','sci_water_pressure':'pressure','sci_water_temp':'temperature',
            'sci_water_cond':'conductivity','sci_water_sal':'salinity','sci_water_dens':'density','sci_flbbcd_bb_units':'turbidity',
            'sci_flbbcd_cdom_units':'cdom','sci_flbbcd_chlor_units':'chlorophyll','sci_oxy4_oxygen':'oxygen'})
        else:
            self.mission_data.ds_sci = self.mission_data.ds_sci.rename({'index': 'time','sci_water_pressure':'pressure','sci_water_temp':'temperature',
            'sci_water_cond':'conductivity','sci_water_sal':'salinity','sci_water_dens':'density'})

    def process_sci_data(self) -> xr.Dataset:
        '''Perform all processing of science data from ascii to pandas dataframe to xarray dataset'''
        print_time('Processing Science Data')
        # Process Science Data
        self.load_science()
        self.convert_sci_df_to_ds()
        self.add_sci_attrs()
        self.format_sci_ds()
        # self.mission_data.get_output_nc_path()
        print_time('Finished Processing Science Data')




import pandas as pd
import xarray as xr
from pathlib import Path
from attrs import define,field,asdict
import datetime
import os
from pprint import pformat


from glider_ingest.utils import find_nth,invert_dict

@define
class MissionData:
    # Required Variables
    memory_card_copy_loc:Path
    working_dir:Path
    mission_num:str

    # Optional Variables
    glider_id:str = field(default=None)
    nc_filename:str = field(default=None)
    output_nc_path:Path = field(default=None)
    mission_start_date:str|None = field(default=None)
    mission_end_date:str|None = field(default=None)
    mission_year:str = field(default=None)
    mission_title:str = field(default=None)
    glider_name:str = field(default=None)
    glider_ids:dict = field(default={'199':'Dora','307':'Reveille','308':'Howdy','540':'Stommel','541':'Sverdrup','1148':'unit_1148'})
    wmo_ids:dict = field(default={'199':'unknown','307':'4801938','308':'4801915','540':'4801916','541':'4801924','1148':'4801915'})

    # Post init variables
    fli_files_loc:Path = field(init=False)
    fli_cache_loc:Path = field(init=False)

    sci_files_loc:Path = field(init=False)
    sci_cache_loc:Path = field(init=False)

    wmo_id:str = field(init=False)

    df_fli:pd.DataFrame = field(init=False)
    ds_fli:xr.Dataset = field(init=False)

    df_sci:pd.DataFrame = field(init=False)
    ds_sci:xr.Dataset = field(init=False)

    ds_mission:xr.Dataset = field(init=False)

    def __attrs_post_init__(self):
        self.get_file_locs()
        self.get_mission_date_range()
        self.get_mission_year_and_glider()
        self.get_wmo_id()
        self.get_mission_title()
        self.get_nc_filename()
        self.get_output_nc_path()
        

    def get_file_locs(self):
        self.fli_files_loc = self.memory_card_copy_loc.joinpath('Flight_card/logs')
        self.fli_cache_loc = self.memory_card_copy_loc.joinpath('Flight_card/state/cache')

        self.sci_files_loc = self.memory_card_copy_loc.joinpath('Science_card/logs')
        self.sci_cache_loc = self.memory_card_copy_loc.joinpath('Science_card/state/cache')

    def get_mission_date_range(self):
        if self.mission_end_date is None:
            self.mission_end_date = str(datetime.datetime.today().date()+datetime.timedelta(days=365))
        if self.mission_start_date is None:
            self.mission_start_date = '2010-01-01'

    def get_mission_year_and_glider(self):
        files = self.get_files(files_loc=self.sci_files_loc,extension='ebd')
        file = files[10]            
        fp = open(file, errors="ignore")
        for line in fp:
            if 'full_filename' in line.strip():
                name = line.replace('full_filename:','').strip()
                self.mission_year = name[name.find('-')+1:find_nth(name,'-',2)].strip()
                self.glider_name = name[:name.find('-')].strip()
                self.glider_name = self.glider_name.replace('unit_','')
                inverted_glider_ids = invert_dict(self.glider_ids)
                # Get the glider_id using the glider_name, sometimes the name given by full_filename is the key and other times its the value
                try:
                    self.glider_name = self.glider_ids[self.glider_name]
                    if self.glider_id is None:
                        self.glider_id = inverted_glider_ids[self.glider_name]
                except KeyError as e:
                    try:
                        self.glider_name = inverted_glider_ids[self.glider_name]
                        if self.glider_id is None:
                            self.glider_id = self.glider_ids[self.glider_name]
                    except KeyError as e:
                        raise ValueError(f'Could not find glider_id, please pass glider_id. Must be one of {list(self.glider_ids.keys())} ({e})')
        # Check if we found the glider_id in the file
        if self.glider_id is None:
            raise ValueError(f'Could not find glider_id, please pass glider_id. Must be one of {list(self.glider_ids.keys())}')
        fp.close()

    def get_mission_title(self):
        if self.mission_title is None:
            self.mission_title = f'Mission {self.mission_num}'

    def get_nc_filename(self):
        if self.nc_filename is None:
            self.nc_filename = f'M{self.mission_num}_{self.mission_year}_{self.glider_id}.nc'

    def get_output_nc_path(self):
        if self.output_nc_path is None:
            output_nc_loc = self.working_dir.joinpath(self.mission_title)
            output_nc_loc.mkdir(exist_ok=True,parents=True)
            self.output_nc_path = output_nc_loc.joinpath(self.nc_filename)
        # Ensure self.output_nc_path is a pathlib.Path object
        elif isinstance(self.output_nc_path,Path):
            # If the provided output_nc_path does not specify the filename
            if not self.output_nc_path.is_file():
                self.output_nc_path.joinpath(self.nc_filename)

    def get_wmo_id(self):
        '''Get glider wmo id from glider id and dict of wmo ids'''
        self.wmo_id = self.wmo_ids[self.glider_id]

    def get_files(self,files_loc:Path,extension:str):
        '''Get files to process from files_loc'''
        if files_loc.exists():
            try:
                files = list(files_loc.rglob(f'*.{extension}'))
                files = [str(file) for file in files]
                if len(files) ==0 :
                    raise ValueError(f'No Files found at {files_loc}')
            except ValueError:
                files = list(files_loc.rglob(f'*.{extension.upper()}'))
                files = [str(file) for file in files]
                if len(files) ==0 :
                    raise ValueError(f'No Files found at {files_loc}')
            return files
        else: 
            raise ValueError(f'Invaid path for files: {files_loc}')
        


import numpy as np
import pandas as pd
import xarray as xr
import uuid
from attrs import define,field
from pathlib import Path

from glider_ingest.MissionData import MissionData
from glider_ingest.ScienceProcessor import ScienceProcessor
from glider_ingest.FlightProcessor import FlightProcessor
from glider_ingest.utils import add_gridded_data,get_polygon_coords


@define
class MissionProcessor:

    mission_data:MissionData

    def add_global_attrs(self) -> xr.Dataset:
        '''Add attributes to the mission dataset'''
        self.mission_data.ds_mission.attrs = {'Conventions': 'CF-1.6, COARDS, ACDD-1.3',
        'acknowledgment': ' ',
        'cdm_data_type': 'Profile',
        'comment': 'time is the ctd_time from sci_m_present_time, m_time is the gps_time from m_present_time, g_time and g_pres are the grided time and pressure',
        'contributor_name': 'Steven F. DiMarco',
        'contributor_role': ' ',
        'creator_email': 'sakib@tamu.edu, gexiao@tamu.edu',
        'creator_institution': 'Texas A&M University, Geochemical and Environmental Research Group',
        'creator_name': 'Sakib Mahmud, Xiao Ge',
        'creator_type': 'persons',
        'creator_url': 'https://gerg.tamu.edu/',
        'date_created': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S'),
        'date_issued': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S'),
        'date_metadata_modified': '2023-09-15',
        'date_modified': pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S'),
        'deployment': ' ',
        'featureType': 'profile',
        'geospatial_bounds_crs': 'EPSG:4326',
        'geospatial_bounds_vertical_crs': 'EPSG:5831',
        'geospatial_lat_resolution': "{:.4e}".format(abs(np.nanmean(np.diff(self.mission_data.ds_mission.latitude))))+ ' degree',
        'geospatial_lat_units': 'degree_north',
        'geospatial_lon_resolution': "{:.4e}".format(abs(np.nanmean(np.diff(self.mission_data.ds_mission.longitude))))+ ' degree',
        'geospatial_lon_units': 'degree_east',
        'geospatial_vertical_positive': 'down',
        'geospatial_vertical_resolution': ' ',
        'geospatial_vertical_units': 'EPSG:5831',
        'infoUrl': 'https://gerg.tamu.edu/',
        'institution': 'Texas A&M University, Geochemical and Environmental Research Group',
        'instrument': 'In Situ/Laboratory Instruments > Profilers/Sounders > CTD',
        'instrument_vocabulary': 'NASA/GCMD Instrument Keywords Version 8.5',
        'ioos_regional_association': 'GCOOS-RA',
        'keywords': 'Oceans > Ocean Pressure > Water Pressure, Oceans > Ocean Temperature > Water Temperature, Oceans > Salinity/Density > Conductivity, Oceans > Salinity/Density > Density, Oceans > Salinity/Density > Salinity',
        'keywords_vocabulary': 'NASA/GCMD Earth Sciences Keywords Version 8.5',
        'license': 'This data may be redistributed and used without restriction.  Data provided as is with no expressed or implied assurance of quality assurance or quality control',
        'metadata_link': ' ',
        'naming_authority': 'org.gcoos.gandalf',
        'ncei_template_version': 'NCEI_NetCDF_Trajectory_Template_v2.0',
        'platform': 'In Situ Ocean-based Platforms > AUVS > Autonomous Underwater Vehicles',
        'platform_type': 'Slocum Glider',
        'platform_vocabulary': 'NASA/GCMD Platforms Keywords Version 8.5',
        'processing_level': 'Level 0',
        'product_version': '0.0',
        'program': ' ',
        'project': ' ',
        'publisher_email': 'sdimarco@tamu.edu',
        'publisher_institution': 'Texas A&M University, Geochemical and Environmental Research Group',
        'publisher_name': 'Steven F. DiMarco',
        'publisher_url': 'https://gerg.tamu.edu/',
        'references': ' ',
        'sea_name': 'Gulf of Mexico',
        'standard_name_vocabulary': 'CF Standard Name Table v27',
        'summary': 'Merged dataset for GERG future usage.',
        'time_coverage_resolution': ' ',
        'wmo_id': self.mission_data.wmo_id,
        'uuid': str(uuid.uuid4()),
        'history': 'dbd and ebd files transferred from dbd2asc on 2023-09-15, merged into single netCDF file on '+pd.Timestamp.now().strftime(format='%Y-%m-%d %H:%M:%S'),
        'title': self.mission_data.mission_title,
        'source': 'Observational Slocum glider data from source ebd and dbd files',
        'geospatial_lat_min': str(np.nanmin(self.mission_data.ds_mission.latitude[np.where(self.mission_data.ds_mission.latitude.values<29.5)].values)),
        'geospatial_lat_max': str(np.nanmax(self.mission_data.ds_mission.latitude[np.where(self.mission_data.ds_mission.latitude.values<29.5)].values)),
        'geospatial_lon_min': str(np.nanmin(self.mission_data.ds_mission.longitude.values)),
        'geospatial_lon_max': str(np.nanmax(self.mission_data.ds_mission.longitude.values)),
        'geospatial_bounds': get_polygon_coords(self.mission_data.ds_mission),
        'geospatial_vertical_min': str(np.nanmin(self.mission_data.ds_mission.depth[np.where(self.mission_data.ds_mission.depth>0)].values)),
        'geospatial_vertical_max': str(np.nanmax(self.mission_data.ds_mission.depth.values)),
        'time_coverage_start': str(self.mission_data.ds_mission.time[-1].values)[:19],
        'time_coverage_end': str(self.mission_data.ds_mission.m_time[-1].values)[:19],
        'time_coverage_duration': 'PT'+str((self.mission_data.ds_mission.m_time[-1].values - self.mission_data.ds_mission.time[-1].values) / np.timedelta64(1, 's'))+'S'}


    def process_sci(self):
        sci_processor = ScienceProcessor(mission_data=self.mission_data)
        sci_processor.process_sci_data()
        return sci_processor.mission_data

    def process_fli(self):
        fli_processor = FlightProcessor(mission_data=self.mission_data)
        fli_processor.process_flight_data()
        return fli_processor.mission_data
    
    def generate_mission_dataset(self):
        self.mission_data = self.process_sci()
        self.mission_data = self.process_fli()

        # Combine sci and fli datasets
        self.mission_data.ds_mission = self.mission_data.ds_sci.copy()
        self.mission_data.ds_mission.update(self.mission_data.ds_fli)

        # Add gridded data to mission dataset
        self.mission_data.ds_mission = add_gridded_data(self.mission_data.ds_mission)

        # Add attributes to the mission dataset
        self.add_global_attrs()

    def save_mission_dataset(self):
        if not hasattr(self.mission_data,'ds_mission'):
            self.generate_mission_dataset()
        self.mission_data.ds_mission.to_netcdf(self.mission_data.output_nc_path,engine='netcdf4')
